/*
 *	check implementation/understanding of tt bank switching feature
 *
 *	usage:	banksw
 */
#include <stdio.h>
#include <stdlib.h>

#include <cookie.h>
#include <osbind.h>
#include <linea.h>
#include <conio.h>

#define VERSION			"0.9"

#define TT_VIDEO		0x00020000

#define TT_SHIFTER		0xffff8262L	/* manifest constants */
#define TT_PALETTE_REGS	0xffff8400L

/*
 *	global
 */
long cookie = 0L;
int maxcolour, rez;
unsigned short save_tt_shifter;
unsigned short save_tt_palette[256];

/*
 *	function prototypes
 */
void check_palette(int bank);
void draw_box(int colour);
//void get_tt_palette(short *palette);
//unsigned short get_tt_shifter(void);
void more(int clear);
void quit(int rc);
void restore_state(void);
void save_state(void);
void test(void);


int main(int argc,char **argv)
{
	printf("banksw v%s start\n",VERSION);

	getcookie(_VDO,&cookie);
	if (cookie != TT_VIDEO) {
		fprintf(stderr,"This program only runs on a TT!\n");
		quit(1);
	}

	switch(rez=Getrez()) {
	case 0:		/* ST low */
	case 4:		/* TT medium */
		maxcolour = 16;
		break;
	case 1:		/* ST medium */
		maxcolour = 4;
		break;
	default:
		fprintf(stderr,"This program requires ST low, ST medium, or TT medium!\n");
		quit(1);
	}

	save_state();	/* save tt shifter, palette */
	more(1);

	test();

	restore_state();	/* restore tt shifter, palette */

	printf("banksw v%s end\n",VERSION);

	quit(0);
	return 0;
}

/* test routine */
void test(void)
{
	int i;
	short tt_shifter, bank;

	linea0();	/* initialise */

	for (i = 0; i < maxcolour; i++)
		draw_box(i);

	fprintf(stderr,"\n\n\n\n\n\n\n\n\n\n\n");

	tt_shifter = EgetShift();
	bank = tt_shifter & 0x000f;
	if (bank != 0)
		fprintf(stderr,"Initial bank is not zero! (actually %d)\n",bank);

	for (bank = 0; bank < 16; bank++) {
		EsetBank(bank);
		fprintf(stderr,"Check bank%d...",bank);
		check_palette(bank);
		more(0);
	}
}

void check_palette(int bank)
{
	int i, error;
	short ste_colour, tt_colour, temp;

	for (i = 0, error = 0; i < maxcolour; i++) {
		ste_colour = Setcolor(i,-1);
		tt_colour = EsetColor(bank*16+i,-1);
		temp = ((ste_colour&0x0777)<<1) | ((ste_colour&0x0888)>>3);
		if (temp != tt_colour) {
//			fprintf(stderr,"\nSTe colour %d (0x%04x) does not match TT colour %d (0x%04x)",
//					i,ste_colour,bank*16+i,tt_colour);
			error++;
		}
	}
	if (error)
		fprintf(stderr,"%d errors",error);
	else fprintf(stderr,"OK");
}

void draw_box(int colour)
{
	short pattern = 0xffff;
	int depth = (rez==0)?5:10;
	int width = (rez==0)?100:200;

	X1 = 0;
	Y1 = X1 + colour*depth;
	X2 = X1 + width;
	Y2 = Y1 + depth-1;
	COLBIT0 = colour & 0x01;
	COLBIT1 = (colour>>1) & 0x01;
	COLBIT2 = (colour>>2) & 0x01;
	COLBIT3 = (colour>>3) & 0x01;
	WMODE = 0;
	PATPTR = &pattern;
	PATMSK = 0;
	MFILL = 0;
	CLIP = 0;

	linea5();
}

void more(int clear)
{
	fprintf(stderr,"...press enter\r");
	getch();	/* no echo of return key */
	if (clear)
		fprintf(stderr,"\033E");
	else fprintf(stderr,"\033K");
}

void quit(int rc)
{
	fprintf(rc?stderr:stdout,"Press a key to exit program\n");
	getchar();

	exit(rc);
}

void restore_state()
{
char *save;
volatile unsigned short *pal_regs = (unsigned short *)TT_PALETTE_REGS;
unsigned short *q = save_tt_palette;
int i;

	save = Super(NULL);
	*(volatile unsigned short *)TT_SHIFTER = save_tt_shifter;
	for (i = 0; i < 256; i++)
		*pal_regs++ = *q++;
	Super(save);
}

void save_state()
{
char *save;
volatile unsigned short *pal_regs = (unsigned short *)TT_PALETTE_REGS;
unsigned short *q = save_tt_palette;
int i;

	save = Super(NULL);
	save_tt_shifter = *(volatile unsigned short *)TT_SHIFTER;
	for (i = 0; i < 256; i++)
		*q++ = *pal_regs++;
	Super(save);
}

#if 0
unsigned short get_tt_shifter()
{
char *save;
short hw;

	save = Super(NULL);
	hw = *(volatile unsigned short *)TT_SHIFTER;
	Super(save);

	return hw;
}

void get_tt_palette(short *palette)
{
char *save;

	save = Super(NULL);
	memcpy(palette,(unsigned short *)TT_PALETTE_REGS,512);
	Super(save);
}
#endif
